# from colav_bridge.ros_to_proto import parse_controller_feedback
# from colav_protobuf.examples import controller_feedback
# from colav_interfaces.msg import ControllerFeedback
# from std_msgs.msg import Header
# from builtin_interfaces.msg import Time
# from colav_interfaces.msg import CmdVelYaw, ControlMode, ControlStatus


# def test_controller_feedback_parser():
#     ros_controller_feedback = ControllerFeedback(
#         header = Header(
#             stamp = Time(
#                 sec = controller_feedback.stamp.sec,
#                 nanosec = controller_feedback.stamp.nanosec 
#             ),
#             frame_id = 'map'
#         ),
#         mission_tag = controller_feedback.mission_tag,
#         agent_tag = controller_feedback.agent_tag,
#         cmd = CmdVelYaw(
#             velocity=float(20),
#             yaw_rate=float(0.2)
#         ),
#         mode = ControlMode(
#             type = ControlMode.CRUISE
#         ),
#         status = ControlStatus(
#             type = ControlStatus.ACTIVE,
#             message = "Hybrid Automaton is ACTIVE"
#         ) 
#     )

#     parsed_controller_feedback = parse_controller_feedback(ros_controller_feedback)

#     assert parsed_controller_feedback.stamp.sec == ros_controller_feedback.header.stamp.sec
#     assert parsed_controller_feedback.stamp.nanosec == ros_controller_feedback.header.stamp.nanosec
#     assert parsed_controller_feedback.mission_tag == ros_controller_feedback.mission_tag
#     assert parsed_controller_feedback.agent_tag == ros_controller_feedback.agent_tag
#     assert round(parsed_controller_feedback.cmd.velocity, 3) == round(ros_controller_feedback.cmd.velocity, 3)
#     assert round(parsed_controller_feedback.cmd.yaw_rate, 3) == round(ros_controller_feedback.cmd.yaw_rate, 3)
#     assert parsed_controller_feedback.mode == ros_controller_feedback.mode.type
#     assert parsed_controller_feedback.status == ros_controller_feedback.status.type
    
#     assert parsed_controller_feedback.__module__ == 'controllerFeedback_pb2'



from colav_interfaces.msg import ControllerFeedback as ROSControllerFeedback
from colav_protobuf_utils.protobuf_generator import gen_controller_feedback, CtrlMode, CtrlStatus
from colav_protobuf import ControllerFeedback as ProtoControllerFeedback

def parse_controller_feedback(msg:ROSControllerFeedback) -> ProtoControllerFeedback:
    """Parse controller feedback to ROS message"""
    try: 

        return gen_controller_feedback(
            mission_tag= msg.mission_tag,
            agent_tag= msg.agent_tag,
            mode= CtrlMode(msg.mode.type),
            status = CtrlStatus(msg.status.type),
            velocity = msg.cmd.velocity,
            yaw_rate = msg.cmd.yaw_rate,
            stamp=msg.header.stamp
        )
    except Exception as e:
        raise ValueError(f"Failed to parse exception: {e}") from e


from geometry_msgs.msg import Polygon, Point32

def parse_polygon(points) -> Polygon:
    """Parse a list of points into a Polygon message."""
    try:
        points = [Point32(x=point.x, y=point.y, z=point.z) for point in points]
        return Polygon(points=points)
    except Exception as e:
        raise ValueError(f"Failed to parse polygon: {e}") from e


from colav_interfaces.msg import StaticObstacleGeometry
from geometry_msgs.msg import Polygon, Point32 

def parse_static_obstacle_geometry(geometry) -> StaticObstacleGeometry:
    """Convert protobuf geometry to ROS StaticObstacleGeometry"""
    try:
        return StaticObstacleGeometry( 
            polyshape = Polygon(points=[Point32(x=point.x, y=point.y, z=point.z) for point in geometry.polyshape_points]),
            inflation_radius = geometry.inflation_radius
        )
    except Exception as e: 
        raise ValueError("Error parsing static obstacle geometry") from e


from geometry_msgs.msg import Point32

def parse_point(point):
    """Parse a protobuf point to ros."""
    try: 
        return Point32(
            x=point.x,
            y=point.y,
            z=point.z
        )
    except Exception as e: 
        raise ValueError("Error parsing point") from e



from colav_bridge.proto_to_ros.utils import parse_point
from colav_protobuf import MissionRequest

def test_parse_point():
    point = MissionRequest.Point(x=1, y=2, z=2)
    parsed_point = parse_point(point)
    assert parsed_point.x == 1
    assert parsed_point.y == 2
    assert parsed_point.z == 2
    assert parsed_point.__class__.__name__ == 'Point32'
    assert parsed_point.__module__ == 'geometry_msgs.msg._point32'
    
    
# TODO: Exception test
def test_parse_point_exception():
    pass


from rclpy.node import Node
from colav_interfaces.msg import AgentUpdate
from colav_protobuf_utils import ProtoType
from colav_protobuf import AgentUpdate as AgentUpdateProto
from colav_protobuf_utils.deserialization import deserialize_protobuf
from std_msgs.msg import Header
from .utils import parse_pose, parse_stamp
from builtin_interfaces.msg import Time

def parse_agent_update(msg: bytes) -> AgentUpdate:
    """Parse agent update protobuf to ROS"""
    try:
        protobuf_agent_update = deserialize_protobuf(msg, ProtoType.AGENT_UPDATE)
        return AgentUpdate(
            header = Header(
                stamp = parse_stamp(protobuf_agent_update.stamp),
                frame_id = "map"
            ),
            mission_tag=protobuf_agent_update.mission_tag,
            agent_tag=protobuf_agent_update.agent_tag,
            pose=parse_pose(protobuf_agent_update.state.pose),
            velocity=protobuf_agent_update.state.velocity,
            acceleration=protobuf_agent_update.state.acceleration,
            yaw_rate=protobuf_agent_update.state.yaw_rate
        )
    except Exception as e:
        raise ValueError(f"Error parsing agent protobuf: {e}") from e


from colav_bridge.proto_to_ros.utils import parse_polygon
from colav_protobuf import ObstaclesUpdate

def test_parse_polygon():
    point = ObstaclesUpdate.Point
    polyshape = [
        point(x=float(1), y=float(2), z=float(3)),
        point(x=float(4), y=float(5), z=float(6))
    ]
    
    parsed_polyshape = parse_polygon(polyshape)
    
    assert len(parsed_polyshape.points) == 2
    assert parsed_polyshape.points[0].x == 1
    assert parsed_polyshape.points[0].y == 2
    assert parsed_polyshape.points[0].z == 3
    assert parsed_polyshape.points[1].x == 4
    assert parsed_polyshape.points[1].y == 5
    assert parsed_polyshape.points[1].z == 6
    assert parsed_polyshape.__module__ == 'geometry_msgs.msg._polygon'

# TODO: Add test for exception
def test_parse_polygon_exception():
    pass


from colav_bridge.proto_to_ros.utils import parse_pose
from colav_protobuf import AgentUpdate

def test_parse_pose():
    pose = AgentUpdate.Pose(position=AgentUpdate.Pose.Position(x=1, y=2, z=2), orientation=AgentUpdate.Pose.Orientation(x=1, y=2, z=2, w=1))
    parsed_pose = parse_pose(pose)
    assert parsed_pose.position.x == 1
    assert parsed_pose.position.y == 2
    assert parsed_pose.position.z == 2
    assert parsed_pose.orientation.x == 1
    assert parsed_pose.orientation.y == 2
    assert parsed_pose.orientation.z == 2
    assert parsed_pose.orientation.w == 1
    assert parsed_pose.__module__ == 'geometry_msgs.msg._pose'
    


from colav_bridge.proto_to_ros.utils import parse_vessel
from colav_protobuf import MissionRequest

def test_parse_vessel():
    vessel = MissionRequest.Vessel(
        tag = "EF12_WORKBOAT",
        type = MissionRequest.Vessel.VesselType.HYDROFOIL,
        constraints = MissionRequest.Vessel.VesselConstraints(
            max_acceleration = 2.0,
            max_deceleration = -1.0,
            max_velocity = 30.0,
            min_velocity = 15.0,
            max_yaw_rate = 0.2
        ),
        geometry = MissionRequest.Vessel.VesselGeometry (
            safety_radius = 5,
            loa = 12.0,
            beam = 4.0
        )
    )
    parsed_vessel = parse_vessel(vessel)
    
    assert parsed_vessel.tag == "EF12_WORKBOAT"
    assert MissionRequest.Vessel.VesselType.Value(parsed_vessel.type) == MissionRequest.Vessel.VesselType.HYDROFOIL
    assert round(parsed_vessel.constraints.max_acceleration, 3) == 2.00
    assert round(parsed_vessel.constraints.max_deceleration, 3) == -1.00
    assert round(parsed_vessel.constraints.max_velocity, 3) == 30.0
    assert round(parsed_vessel.constraints.min_velocity, 3) == 15.0
    assert round(parsed_vessel.constraints.max_yaw_rate, 3) == 0.200
    assert round(parsed_vessel.geometry.safety_radius, 3) == 5.00
    assert round(parsed_vessel.geometry.loa, 3) == 12.0
    assert round(parsed_vessel.geometry.beam, 3) == 4.00
    assert parsed_vessel.__module__ == 'colav_interfaces.msg._vessel'


from rclpy.node import Node
from colav_interfaces.msg import ObstaclesUpdate, DynamicObstacle, StaticObstacle
from colav_protobuf import ObstaclesUpdate as ProtoObstaclesUpdate
from .utils import parse_point, parse_pose, parse_dynamic_obstacle_geometry, parse_static_obstacle_geometry, parse_stamp
from std_msgs.msg import Header
from colav_protobuf_utils.deserialization import deserialize_protobuf
from colav_protobuf_utils import ProtoType
from builtin_interfaces.msg import Time

def parse_obstacles_update(msg: bytes) -> ObstaclesUpdate:
    """Parse Obstacle update received via protobuf and publish it to ros topic"""
    try:
        protobuf_obstacles_update = deserialize_protobuf(msg, ProtoType.OBSTACLES_UPDATE)
        return ObstaclesUpdate(
            header=Header(
                stamp = parse_stamp(protobuf_obstacles_update.stamp),
                frame_id = "map"
            ),
            mission_tag=protobuf_obstacles_update.mission_tag,
            dynamic_obstacles=_parse_dynamic_obstacles(protobuf_obstacles_update.dynamic_obstacles),
            static_obstacles=_parse_static_obstacles(protobuf_obstacles_update.static_obstacles),
        )
    except Exception as e:
        raise ValueError(f"Error parsing obstacles protobuf: {e}") from e
    
def _parse_dynamic_obstacles(dynamic_obstacles) -> list[DynamicObstacle]:
    """Convert dynamic obstacles from protobuf to ROS"""
    try: 
        return [
            DynamicObstacle(
                tag=dynamic_obstacle.tag,
                type=ProtoObstaclesUpdate.DynamicObstacleType.Name(dynamic_obstacle.type),
                pose=parse_pose(dynamic_obstacle.state.pose),
                velocity=dynamic_obstacle.state.velocity,
                acceleration=float(0), # TODO: Add acceleration to protobuf for future reference
                yaw_rate=dynamic_obstacle.state.yaw_rate,
                geometry=parse_dynamic_obstacle_geometry(dynamic_obstacle.geometry),
            )
            for dynamic_obstacle in dynamic_obstacles
        ]
    except Exception as e:
        raise ValueError(f"Error parsing dynamic obstacles: {e}") from e
    
def _parse_static_obstacles(static_obstacles) -> list[StaticObstacle]:
    """Convert static obstacles from protobuf to ROS"""
    try: 
        return [
            StaticObstacle(
                tag=static_obstacle.tag,
                type=ProtoObstaclesUpdate.StaticObstacleType.Name(static_obstacle.type),
                pose=parse_pose(static_obstacle.pose),
                geometry=parse_static_obstacle_geometry(static_obstacle.geometry),
            )
            for static_obstacle in static_obstacles
        ]
    except Exception as e:
        raise ValueError(f"Error parsing static obstacles: {e}") from e



from colav_bridge.proto_to_ros import parse_obstacles_update
from colav_protobuf.examples import obstacles_update
from colav_protobuf_utils.serialization import serialize_protobuf

def test_parse_obstacles_update():
    parsed_obstacles_update = parse_obstacles_update(serialize_protobuf(obstacles_update))    
    
    assert parsed_obstacles_update.mission_tag == obstacles_update.mission_tag
    assert parsed_obstacles_update.header.stamp.sec == obstacles_update.stamp.sec
    assert parsed_obstacles_update.header.stamp.nanosec == obstacles_update.stamp.nanosec
    assert parsed_obstacles_update.header.frame_id == 'map'

    # validate parsing of static obstacles
    assert len(parsed_obstacles_update.static_obstacles) == len(obstacles_update.static_obstacles)
    for idx in range(len(parsed_obstacles_update.static_obstacles)):
        parsed_static_obstacle = parsed_obstacles_update.static_obstacles[idx]
        static_obstacle = obstacles_update.static_obstacles[idx]

        assert parsed_static_obstacle.tag == static_obstacle.tag
        # assert parsed_static_obstacle.type == static_obstacle.type # TODO: Need to fix this
        assert parsed_static_obstacle.pose.position.x == static_obstacle.pose.position.x
        assert parsed_static_obstacle.pose.position.y == static_obstacle.pose.position.y
        assert parsed_static_obstacle.pose.position.z == static_obstacle.pose.position.z

        assert parsed_static_obstacle.pose.orientation.x == static_obstacle.pose.orientation.x
        assert parsed_static_obstacle.pose.orientation.y == static_obstacle.pose.orientation.y
        assert parsed_static_obstacle.pose.orientation.z == static_obstacle.pose.orientation.z
        assert parsed_static_obstacle.pose.orientation.w == static_obstacle.pose.orientation.w
        # TODO: NEED TO CHECK POLYSHAPE
        assert parsed_static_obstacle.geometry.inflation_radius == static_obstacle.geometry.inflation_radius


    # validate parsing of dynamic obstacles
    assert len(parsed_obstacles_update.dynamic_obstacles) == len(obstacles_update.dynamic_obstacles)
    for idx in range(len(parsed_obstacles_update.dynamic_obstacles)):
        parsed_dynamic_obstacle = parsed_obstacles_update.dynamic_obstacles[idx]
        dynamic_obstacle = obstacles_update.dynamic_obstacles[idx]

        assert parsed_dynamic_obstacle.tag == dynamic_obstacle.tag
        # assert parsed_static_obstacle.type == static_obstacle.type # TODO: Need to fix this
        assert parsed_dynamic_obstacle.pose.position.x == dynamic_obstacle.state.pose.position.x
        assert parsed_dynamic_obstacle.pose.position.y == dynamic_obstacle.state.pose.position.y
        assert parsed_dynamic_obstacle.pose.position.z == dynamic_obstacle.state.pose.position.z

        assert parsed_dynamic_obstacle.pose.orientation.x == dynamic_obstacle.state.pose.orientation.x
        assert parsed_dynamic_obstacle.pose.orientation.y == dynamic_obstacle.state.pose.orientation.y
        assert parsed_dynamic_obstacle.pose.orientation.z == dynamic_obstacle.state.pose.orientation.z
        assert parsed_dynamic_obstacle.pose.orientation.w == dynamic_obstacle.state.pose.orientation.w

        assert parsed_dynamic_obstacle.velocity == dynamic_obstacle.state.velocity
        assert parsed_dynamic_obstacle.yaw_rate == dynamic_obstacle.state.yaw_rate


    assert parsed_obstacles_update.__module__ == 'colav_interfaces.msg._obstacles_update'


from .parse_mission_request import parse_mission_request
from .parse_agent_update import parse_agent_update
from .parse_obstacles_update import parse_obstacles_update

__all__ = [
    "parse_mission_request",
    "parse_agent_update",
    "parse_obstacles_update"
]



from colav_bridge.proto_to_ros.utils import parse_dynamic_obstacle_geometry
from colav_protobuf import ObstaclesUpdate

def test_parse_dynamic_obstacle_geometry():
    obstacle_geometry = ObstaclesUpdate.DynamicObstacleGeometry (
        loa = float(10),
        beam = float(5),
        safety_radius  = float(10)
    )
    
    parsed_obstacle_geometry = parse_dynamic_obstacle_geometry(
        obstacle_geometry
    )
    
    assert parsed_obstacle_geometry.loa == 10
    assert parsed_obstacle_geometry.beam == 5
    assert parsed_obstacle_geometry.safety_radius == 10
    assert parsed_obstacle_geometry.__module__ == 'colav_interfaces.msg._dynamic_obstacle_geometry'
    
# TODO: Add test for exception
def test_parse_dynamic_obstacle_geometry_exception():
    pass


# from colav_bridge.proto_to_ros import parse_mission_request
# from colav_protobuf.examples import mission_request
# from colav_protobuf_utils.serialization import serialize_protobuf
# from colav_protobuf_utils import VesselType

# def test_parse_mission_request():
#     parsed_mission_request = parse_mission_request(serialize_protobuf(mission_request))
    
#     assert parsed_mission_request.tag == mission_request.tag
#     assert parsed_mission_request.header.stamp.sec == mission_request.stamp.sec
#     assert parsed_mission_request.header.stamp.nanosec == mission_request.stamp.nanosec
#     assert parsed_mission_request.header.frame_id == 'map'
    
#     assert parsed_mission_request.vessel.tag == mission_request.vessel.tag
#     # assert parsed_mission_request.vessel.type == VesselType[mission_request.vessel.type] # TODO: FIX THSI
#     assert parsed_mission_request.vessel.constraints.max_acceleration == mission_request.vessel.constraints.max_acceleration 
#     assert parsed_mission_request.vessel.constraints.max_deceleration == mission_request.vessel.constraints.max_deceleration 
#     assert parsed_mission_request.vessel.constraints.max_velocity == mission_request.vessel.constraints.max_velocity 
#     assert parsed_mission_request.vessel.constraints.min_velocity == mission_request.vessel.constraints.min_velocity 
#     assert parsed_mission_request.vessel.constraints.max_yaw_rate == mission_request.vessel.constraints.max_yaw_rate 
#     assert parsed_mission_request.vessel.geometry.safety_radius == mission_request.vessel.geometry.safety_radius
#     assert parsed_mission_request.vessel.geometry.loa == mission_request.vessel.geometry.loa
#     assert parsed_mission_request.vessel.geometry.beam == mission_request.vessel.geometry.beam

#     assert parsed_mission_request.init_position.x == mission_request.init_position.x
#     assert parsed_mission_request.init_position.y == mission_request.init_position.y
#     assert parsed_mission_request.init_position.z == mission_request.init_position.z

#     assert parsed_mission_request.goal_waypoints[0].position.x == mission_request.goal_waypoints[0].position.x
#     assert parsed_mission_request.goal_waypoints[0].position.y == mission_request.goal_waypoints[0].position.y
#     assert parsed_mission_request.goal_waypoints[0].position.z == mission_request.goal_waypoints[0].position.z
#     assert parsed_mission_request.goal_waypoints[0].acceptance_radius == mission_request.goal_waypoints[0].acceptance_radius

#     assert parsed_mission_request.__module__ == 'colav_interfaces.msg._mission_request'


from colav_bridge.proto_to_ros.utils import parse_waypoints
from colav_protobuf import MissionRequest

def test_parse_waypoints():
    waypoint = MissionRequest.GoalWaypoint (
        position = MissionRequest.Point(
            x = float(1),
            y = float(2),
            z = float(3)
        ),
        acceptance_radius = float(10)
    )
    
    waypoints = [waypoint]
    parsed_waypoints = parse_waypoints(waypoints)
    assert len(parsed_waypoints) == 1
    assert parsed_waypoints[0].position.x == 1
    assert parsed_waypoints[0].position.y == 2
    assert parsed_waypoints[0].position.z == 3
    assert parsed_waypoints[0].acceptance_radius == 10
    
    assert parsed_waypoints[0].__module__ == 'colav_interfaces.msg._waypoint'



from colav_protobuf_utils import ProtoType
from colav_protobuf_utils.deserialization import deserialize_protobuf
from rclpy.node import Node
from colav_interfaces.msg import MissionRequest, Waypoints
from std_msgs.msg import Header
from .utils import parse_vessel, parse_point, parse_waypoints, parse_stamp
from builtin_interfaces.msg import Time

def parse_mission_request(msg: bytes) -> MissionRequest:
    """Parse mission request protobuf to ros"""
    try: 
        protobuf_mission_request = deserialize_protobuf(msg, ProtoType.MISSION_REQUEST)
        return MissionRequest(
                stamp = parse_stamp(protobuf_mission_request.stamp),
                goal_waypoints = Waypoints(waypoints=parse_waypoints((list(protobuf_mission_request.goal_waypoints)))),
        )
    except Exception as e: 
        raise ValueError(f"Error parsing mission request: {e}") from e


from colav_interfaces.msg import Vessel, VesselConstraints, VesselGeometry
from colav_protobuf.missionRequest_pb2 import MissionRequest

def parse_vessel(vessel):
    """Parse vessel protobuf to ros"""
    try:
        return Vessel(
            tag = vessel.tag,
            type = MissionRequest.Vessel.VesselType.Name(
                vessel.type
            ),
            constraints = VesselConstraints(
                max_acceleration = vessel.constraints.max_acceleration,
                max_deceleration = vessel.constraints.max_deceleration,
                max_velocity = vessel.constraints.max_velocity,
                min_velocity = vessel.constraints.min_velocity,
                max_yaw_rate = vessel.constraints.max_yaw_rate,
            ),
            geometry = VesselGeometry(
                loa = vessel.geometry.loa, #ProtoToROSUtils._parse_polygon(vessel.vessel_geometry.polyshape_points),
                beam = vessel.geometry.beam,
                safety_radius = vessel.geometry.safety_radius 
            )
        )
    except Exception as e:
        raise ValueError(f"Error parsing vessel: {e}") from e


from colav_bridge.proto_to_ros import parse_agent_update
from colav_protobuf.examples import agent_update
from colav_protobuf_utils.serialization import serialize_protobuf
from colav_protobuf_utils import ProtoType
from rclpy.node import Node

def test_parse_agent_update():
    parsed_agent_update = parse_agent_update(serialize_protobuf(agent_update))

    assert parsed_agent_update.mission_tag == agent_update.mission_tag
    assert parsed_agent_update.agent_tag == agent_update.agent_tag
    assert parsed_agent_update.header.stamp.sec == agent_update.stamp.sec
    assert parsed_agent_update.header.stamp.nanosec == agent_update.stamp.nanosec
    assert parsed_agent_update.header.frame_id == 'map'

    assert parsed_agent_update.pose.position.x == agent_update.state.pose.position.x
    assert parsed_agent_update.pose.position.y == agent_update.state.pose.position.y
    assert parsed_agent_update.pose.position.z == agent_update.state.pose.position.z
    
    assert parsed_agent_update.pose.orientation.x == agent_update.state.pose.orientation.x
    assert parsed_agent_update.pose.orientation.y == agent_update.state.pose.orientation.y
    assert parsed_agent_update.pose.orientation.z == agent_update.state.pose.orientation.z
    assert parsed_agent_update.pose.orientation.w == agent_update.state.pose.orientation.w

    assert parsed_agent_update.velocity == agent_update.state.velocity
    assert parsed_agent_update.yaw_rate == agent_update.state.yaw_rate
    assert parsed_agent_update.acceleration == agent_update.state.acceleration

    assert parsed_agent_update.__module__ == 'colav_interfaces.msg._agent_update'


from .parse_point import parse_point
from .parse_waypoints import parse_waypoints
from .parse_vessel import parse_vessel
from .parse_polygon import parse_polygon
from .parse_pose import parse_pose
from .parse_static_obstacle_geometry import parse_static_obstacle_geometry
from .parse_dynamic_obstacle_geometry import parse_dynamic_obstacle_geometry
from .parse_stamp import parse_stamp

__all__ = [
    "parse_point",
    "parse_waypoints",
    "parse_vessel",
    "parse_polygon",
    "parse_pose",
    "parse_static_obstacle_geometry",
    "parse_dynamic_obstacle_geometry",
    "parse_stamp"
]


from builtin_interfaces.msg import Time

def parse_stamp(protobuf_stamp) -> Time:
    """Parse a protobuf timestamp to ros."""
    try: 
        return Time(
            sec = protobuf_stamp.sec,
            nanosec = protobuf_stamp.nanosec
        )
    except Exception as e: 
        raise ValueError(f"Error parsing stamp: {e}") from e


from colav_bridge.proto_to_ros.utils import parse_static_obstacle_geometry
from colav_protobuf import ObstaclesUpdate

def test_parse_static_obstacle_geometry():
    obstacle_geometry = ObstaclesUpdate.StaticObstacleGeometry(
        inflation_radius = float(10)
    )
    obstacle_geometry.polyshape_points.extend([
        ObstaclesUpdate.Point(x=float(1), y=float(2), z=float(3)),
        ObstaclesUpdate.Point(x=float(4), y=float(5), z=float(6))
    ])

    parsed_static_obstacle_geom = parse_static_obstacle_geometry(
        obstacle_geometry
    )
    
    assert parsed_static_obstacle_geom.inflation_radius == 10
    assert len(parsed_static_obstacle_geom.polyshape.points) == 2
    assert parsed_static_obstacle_geom.polyshape.points[0].x == 1
    assert parsed_static_obstacle_geom.polyshape.points[0].y == 2
    assert parsed_static_obstacle_geom.polyshape.points[0].z == 3
    assert parsed_static_obstacle_geom.polyshape.points[1].x == 4
    assert parsed_static_obstacle_geom.polyshape.points[1].y == 5
    assert parsed_static_obstacle_geom.polyshape.points[1].z == 6
    assert parsed_static_obstacle_geom.__module__ == 'colav_interfaces.msg._static_obstacle_geometry'


from geometry_msgs.msg import Pose, Point, Quaternion

def parse_pose(pose) -> Pose:
    """Convert protobuf pose to ROS msg"""
    try:
        return Pose(
            position=Point(x=pose.position.x, y=pose.position.y, z=pose.position.z),
            orientation=Quaternion(x=pose.orientation.x, y=pose.orientation.y, z=pose.orientation.z, w=pose.orientation.w)
        )
    except Exception as e:
        raise ValueError("Error parsing pose") from e


from colav_interfaces.msg import Waypoint
from .parse_point import parse_point

def parse_waypoints(waypoints):
    """Parse a list of protobuf waypoints to ros."""
    try:
        waypoints_list = []
        for waypoint in waypoints:
            waypoints_list.append(
                Waypoint(
                    position=parse_point(waypoint.position),
                    acceptance_radius=waypoint.acceptance_radius
                )
            ) 
            
        return waypoints_list
    except Exception as e:
        raise ValueError(f"Error parsing waypoints: {e}") from e



from geometry_msgs.msg import Polygon
from colav_interfaces.msg import DynamicObstacleGeometry

def parse_dynamic_obstacle_geometry(geometry) -> Polygon:
    """Convert protobuf geometry to ROS Polygon"""
    try:
        return DynamicObstacleGeometry(
            loa = geometry.loa,
            beam = geometry.beam,
            safety_radius = geometry.safety_radius
        )
    except Exception as e: 
        raise ValueError("Error parsing dynamic obstacle geometry") from e
    
